import java.io.*;
import java.net.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

public class ChatClient extends JFrame implements Runnable, WindowListener, ActionListener {
	protected String host;
	protected int port;
	protected String username;
	protected JTextArea output;
	protected JTextField input;
	protected DataInputStream dataIn;
	protected DataOutputStream dataOut;
	protected Thread listener;

	public ChatClient(String host, int port, String username) {
		super("ChatClient [" + host + ':' + port + "]" +
				" (" + username + ")");
		this.host = host;
		this.port = port;
		this.username = username;
	}

	public synchronized void start() throws IOException {
		if (listener == null) {
			Socket socket = new Socket(host, port);
			try {
				dataIn = new DataInputStream(
						new BufferedInputStream(
							socket.getInputStream()));
				dataOut = new DataOutputStream(
						new BufferedOutputStream(
							socket.getOutputStream()));
			} catch (IOException ex) {
				socket.close();
				throw ex;
			}
			listener = new Thread(this);
			listener.start();
			makeChatUI();
		}
	}

	public synchronized void stop() throws IOException {
		hide();
		if (listener != null) {
			listener.interrupt();
			listener = null;
			dataOut.close();
		}
	}

	public void run() {
		try {
			while (!Thread.interrupted()) {
				String line = dataIn.readUTF();
				output.append(line + "\n");
			}
		} catch (IOException ex) {
			handleIOException(ex);
		}
	}

	protected void makeChatUI() {
		input = new JTextField(30);
		input.addActionListener(this);
		output = new JTextArea(8,30);
		output.setEditable(false);
		Container c = getContentPane();
		c.add(new JScrollPane(output), BorderLayout.CENTER);
		c.add(input, BorderLayout.SOUTH);
		addWindowListener(this);
		pack();
		show();
	}

	protected synchronized void handleIOException(IOException ex) {
		if (listener != null) {
			output.append(ex + "\n");
			input.setVisible(false);
			validate();
			if (listener != Thread.currentThread())
				listener.interrupt();
			listener = null;
			try {
				dataOut.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	public void windowOpened(WindowEvent event) {
		input.requestFocus();
	}

	public void windowClosing(WindowEvent event) {
		try {
			stop();
		} catch (IOException ex) {
			ex.printStackTrace();
		}
	}

	public void windowClosed(WindowEvent event) {}
	public void windowIconified(WindowEvent event) {}
	public void windowDeiconified(WindowEvent event) {}
	public void windowActivated(WindowEvent event) {}
	public void windowDeactivated(WindowEvent event) {}

	public void actionPerformed(ActionEvent event) {
		try {
			input.selectAll();
			dataOut.writeUTF(username+": "+
				event.getActionCommand());
			dataOut.flush();
		} catch (IOException ex) {
			handleIOException(ex);
		}
	}

	public static void main(String[] args) throws IOException {
		if ((args.length != 2) || (args[0].indexOf(':') < 0))
			throw new IllegalArgumentException(
				"Syntax: ChatClient	<host>:<port> <username>");
		int idx = args[0].indexOf(':');
		String host = args[0].substring(0, idx);
		String username = args[1];
		int port = Integer.parseInt(args[0].substring(idx + 1));
		ChatClient client = new ChatClient(host, port, username);
		client.start();
	}
}