import java.net.*;
import java.io.*;
import java.util.*;

public class ChatServer {
	public static void main(String args[]) throws IOException {
		if (args.length != 1)
			throw new IllegalArgumentException("Syntax: ChatServer <port>");
		int port = Integer.parseInt(args[0]);
		ServerSocket server = new ServerSocket(port);
		// listen for connections
		while (true) {
	  	  Socket client = server.accept();
		  System.out.println("New client from " +
		  	client.getInetAddress());
		  // spawn new handler
		  ChatHandler handler = new ChatHandler(client);
		  handler.start();
		}
	}
}

class ChatHandler implements Runnable {
	// the vector of all handlers
	protected static Vector handlers = new Vector();
	protected Socket socket;
	protected DataInputStream dataIn;
	protected DataOutputStream dataOut;
	protected Thread listener;

	public ChatHandler(Socket socket) {
		this.socket = socket;
	}

	public synchronized void start() {
		if (listener == null) {
			try {
				dataIn = new DataInputStream(
					new BufferedInputStream(
						socket.getInputStream()));
				dataOut = new DataOutputStream(
					new BufferedOutputStream(
						socket.getOutputStream()));
				listener = new Thread(this);
				listener.start();
			} catch (IOException ignored) { }
		}
	}

	public synchronized void stop() {
		if (listener != null) {
			try {
				if (listener != Thread.currentThread())
					listener.interrupt();
				listener = null;
				dataOut.close();
			} catch (IOException ignored) { }
		}
	}

	public void run() {
		try {
			// add us to the pool of handlers
			handlers.addElement(this);
			while (!Thread.interrupted()) {
				// read a message and broadcast it
				String message = dataIn.readUTF();
				broadcast(message);
			}
		} catch (EOFException ignored) {
		} catch (IOException ex) {
			if (listener == Thread.currentThread())
				ex.printStackTrace();
		} finally {
			// thread is to exit, so remove us from handler
			handlers.removeElement(this);
		}
		stop();
	}

	protected void broadcast(String message) {
		synchronized (handlers) {
			Enumeration e = handlers.elements();
			while (e.hasMoreElements()) {
				ChatHandler handler =
					(ChatHandler) e.nextElement();
				try {
					handler.dataOut.writeUTF(message);
					handler.dataOut.flush();
				} catch (IOException ex) {
					// the connection may have dropped
					handler.stop();
				}
			}
		}
	}
}
